#include <cdefBF533.h>
#include <ccblkfn.h>
#include <sys/exception.h>
#include "BlinkLED.h"


// ISR Prototypes
//EX_INTERRUPT_HANDLER(EVT_IVG8_Entry);
EX_INTERRUPT_HANDLER(EVT_IVG9_Entry); 

section("sdram0") void Open_Peripherals(void)
{
	//Configure System
  	Setup_Interrupt_Vectors();	// initialize used handlers
  	Configure_SDRAM();			// set up SDRAM for access
	Configure_Flash();			// set flash up for LED access
	Configure_Flags();			// setup EZKIT switches as interrupt-generating inputs
	Configure_Interrupts();
}


/************************************************************************
 * INTERRUPT VECTORS USED												*
 ************************************************************************/
section("sdram0") void Setup_Interrupt_Vectors(void)
{
	*pSIC_IAR2 = 0x77772777;	// Assign 0x2 (IVG9)  To PFA Interrupt
	
//	register_handler (ik_ivg8, EVT_IVG8_Entry); // Default Real Time Clock Handler (Int8)
	register_handler (ik_ivg9, EVT_IVG9_Entry); // User-Assigned PFA Handler (Int9)
}

/************************************************************************
 * SETUP SDRAM SO THAT WE CAN READ AND WRITE IT							*
 ************************************************************************/    
section("sdram0") void Configure_SDRAM(void)
{
	*pEBIU_SDRRC = 0x0817;		//SDRAM Refresh Rate Control Register
	*pEBIU_SDBCTL = 0x0013;		//SDRAM Memory Bank Control Register

	*pEBIU_SDGCTL = 0x0091998D;	// SDRAM Memory Global Control Register
	ssync();
}

/************************************************************************
 * SETUP FLASH SO THAT WE HAVE ACCESS TO LEDs VIA THE FLASH				*
 ************************************************************************/
section("sdram0") void Configure_Flash(void)
{
	/* Initalize EBIU control registers. */	
	*pEBIU_AMBCTL0 = 0x78B078B0;
	*pEBIU_AMBCTL1 = 0x78B078B0;
	*pEBIU_AMGCTL |= 0xF;
	ssync();

// initialize flash A csio regs (port A and B)
    // clear data registers
	*pFlashA_PortB_Data = 0x0;

    // direction control registers
	*pFlashA_PortB_Dir = (unsigned char)0xFFFF;      // set dir=output
	ssync();
}

/************************************************************************
 * SETUP FLAGS TO ACCEPT PUSH-BUTTON INPUTS AS INTERRUPTS				*
 ************************************************************************/
section("sdram0") void Configure_Flags()
{
   *pFIO_INEN = 0x0400;			// PF10 input enable
   *pFIO_EDGE = 0x0400;			// PF10 edge sensitive
   *pFIO_MASKA_D = 0x0400;		// PF10 Interrupt A
   *pFIO_DIR = 0xFBFF;			// PF15-11, 9-0 Outputs   
   *pFIO_FLAG_S = 0x0080;		// set PF7 high for profiling purposes
}

/************************************************************************
 * CONFIGURE REAL-TIME-CLOCK TO 1 HZ TICKS AND ENABLE STOPWATCH			*
 ************************************************************************/
section("sdram0") void Configure_RTC()
{

	*pRTC_PREN = 0x0;
	
	while (!(*pRTC_ISTAT & 0x8000));
	
	*pRTC_SWCNT = 0x0050;			// set StopWatch to (80/32768) seconds
	*pRTC_ISTAT = CLEAR_RTC_IRQS;	// clear any pending IRQs
	*pRTC_ICTL = 0x1;				// set Stopwatch interrupt enable
	
	while (!(*pRTC_ISTAT & 0x8000));

}

/************************************************************************
 * ENABLE WAKE-UP INTERRUPTS, CORE INTERRUPTS, AND SYSTEM INTERRUPTS	*
 ************************************************************************/
section("sdram0") void Configure_Interrupts(void)
{
   *pSIC_IWR = 0x01;				// Enable PLL interrupt for wakeup.
 
   *pSIC_IMASK = 0x00080000;		// Unmask PFA (Bit 19) Interrupt
   									// Enable GPF Interrupt A.
}

